package ac.ed.lurg.demand;

import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.Timestep;
import ac.ed.lurg.country.CompositeCountry;
import ac.ed.lurg.country.CompositeCountryManager;
import ac.ed.lurg.country.SingleCountry;
import ac.ed.lurg.types.CommodityType;
import ac.ed.lurg.types.ModelFitType;
import ac.ed.lurg.utils.LogWriter;

public class DemandManager {

	private DemandCurveManager demandCurveManager;
	private SspManager sspManager;
	private BaseConsumpManager baseConsumpManager;
	private CompositeCountryManager compositeCountryManager;
	private BioenergyDemandManager bioenergyDemandManager;

	private String ssp_scenario;

	public DemandManager (String ssp_scenario, BaseConsumpManager baseConsumpManager, CompositeCountryManager compositeCountryManager) {
		this.ssp_scenario = ssp_scenario;
		this.baseConsumpManager = baseConsumpManager;
		this.compositeCountryManager = compositeCountryManager;
		demandCurveManager = new DemandCurveManager();		
		sspManager = new SspManager();
		bioenergyDemandManager  = new BioenergyDemandManager();
	}

	public Map<CommodityType, Double> getDemand(CompositeCountry cc, int year) {

		if (!ModelConfig.CHANGE_DEMAND_YEAR) 
			year = ModelConfig.BASE_YEAR;

		Map<CommodityType, Double> demandMap = new HashMap<CommodityType, Double>();

		for (SingleCountry c : compositeCountryManager.getAllForCompositeCountry(cc)) {
			SspData baseSspData = sspManager.get(ssp_scenario, ModelConfig.BASE_YEAR, c);
			SspData sd = sspManager.get(ssp_scenario, year, c);
			
			if (baseSspData == null || sd == null) {
				LogWriter.printlnError(String.format("No baseSspData or ssp for %s, baseYr:%d, year:%d, %s, base:%s, sd:%s. Skipping", ssp_scenario, ModelConfig.BASE_YEAR, year, c.getCountryName(), baseSspData, sd));
				continue;
			}

			LogWriter.println("Got ssp data for " + c.getCountryName() + " of " + sd);

			for (CommodityType commodity : CommodityType.getAllItems()) {
				ModelFitType fitType = commodity.isAnimalProduct() ? ModelFitType.LOGISTIC : ModelFitType.LOGLINEAR;
				
				DemandCurve dc = demandCurveManager.get(commodity, fitType);
				double cpc = dc.getConsumptionPc(baseSspData.getGdpPc(), baseConsumpManager.get(c, commodity), sd.getGdpPc());
				double d = cpc * sd.getPopulation();

				double bioenergy = bioenergyDemandManager.getFirstGenBioenergyDemand(c, year, commodity);
				//		LogWriter.println(String.format("Country %s comm %s: %f",  country, commodity, bioenergy));

				double newDemand = (d + bioenergy) * (1.0 + commodity.getSeedWasteRate());
				Double prevDemand = demandMap.get(commodity);

				if (prevDemand != null)
					newDemand += prevDemand.doubleValue();  // aggregate if composite country

				demandMap.put(commodity, newDemand);
			}
		}

		return demandMap;
	}

	public double getSecondGenBioenergyDemand(Timestep timestep) {
		double d = bioenergyDemandManager.getSecondGenBioenergyDemand(timestep.getYear());
		LogWriter.println("Global gen2 bioenergy demand in " + timestep + " is " + d);
		return d;
	}

	/*	public double getPopulation (SingleCountry country, int year) {
		SspData sd = sspManager.get(ssp_scenario, year, country);
		if (sd == null) {
			LogWriter.printlnError("No ssp data for " + ssp_scenario + ", " + year + ", " + country.getCountryName());
			return 0;
		}
		else
			return sd.getPopulation();
	} 

	public String getModelAndScenarioDescription() {
		return fitType + " " + ssp_scenario;
	}
	 */
}
